# coding: utf-8


from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.tokens import default_token_generator
from django.core.urlresolvers import reverse
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.forms import (AuthenticationForm as DjangoAuthenticationForm,
                                       SetPasswordForm as DjangoSetPasswordForm)

from ..messages import send_transaction_mail
from ..widgets import EmailInput
from .models import User


class SignupForm(forms.ModelForm):
    password1 = forms.CharField(label=_('Password'), widget=forms.PasswordInput)
    password2 = forms.CharField(label=_('Password (verify)'), widget=forms.PasswordInput)

    class Meta:
        model = User
        fields = ('name', 'email', 'password1', 'password2')

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")

        if password1 and password2 and password1 != password2:
            raise forms.ValidationError("Passwords don't match")

        return password2

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class AuthenticationForm(DjangoAuthenticationForm):
    username = forms.EmailField(label=_("E-Mail"), max_length=254, widget=EmailInput())
    password = forms.CharField(label=_("Password"), widget=forms.PasswordInput())


class CustomPasswordResetForm(forms.Form):
    email = forms.EmailField(label=_("E-Mail"), max_length=254, widget=EmailInput())

    def save(self, request, template_name):
        email = self.cleaned_data["email"]

        user_model = get_user_model()
        active_users = user_model.objects.filter(email__iexact=email, is_active=True)
        for user in active_users:
            uid = urlsafe_base64_encode(force_bytes(user.pk))
            token = default_token_generator.make_token(user)
            path = reverse("qs_accounts:password_reset_confirm", kwargs={"uidb64": uid, "token": token})

            use_https = request.is_secure()
            context = {
                "use_https": use_https,
                "email": user.email,
                "uid": uid,
                "user_account": user,
                "token": token,
                "path": path,
                "protocol": "https" if use_https else "http",
            }

            send_transaction_mail(user, template_name, request=request, **context)


class SetPasswordForm(DjangoSetPasswordForm):
    new_password1 = forms.CharField(label=_("New password"), widget=forms.PasswordInput())
    new_password2 = forms.CharField(label=_("New password confirmation"), widget=forms.PasswordInput())


class ProfileForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ("name", "email")
